# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

from .views import BadgeListView, BadgeDetailView


urlpatterns = patterns(
    '',

    url(r'^$',
        BadgeListView.as_view(),
        name='badge_list'),

    url(r'^(?P<slug>[\w-]+)/$',
        BadgeDetailView.as_view(),
        name='badge_detail'),
)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration

from ..compat import get_user_model


User = get_user_model()
user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)


class Migration(SchemaMigration):

    def forwards(self, orm):
        db.add_column(u'badgify_badge', 'users_count',
                      self.gf('django.db.models.fields.IntegerField')(default=0),
                      keep_default=False)

    def backwards(self, orm):
        db.delete_column(u'badgify_badge', 'users_count')

    models = {
        user_model_label: {
            'Meta': {
                'object_name': User.__name__,
                'db_table': "'%s'" % User._meta.db_table
            },
            User._meta.pk.attname: (
                'django.db.models.fields.AutoField', [],
                {'primary_key': 'True',
                'db_column': "'%s'" % User._meta.pk.column}
            ),
        },
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'badgify.award': {
            'Meta': {'unique_together': "(('user', 'badge'),)", 'object_name': 'Award'},
            'awarded_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'badge': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'awards'", 'to': "orm['badgify.Badge']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'badges'", 'to': "orm['%s']" % user_orm_label})
        },
        'badgify.badge': {
            'Meta': {'object_name': 'Badge'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'manual_assignment': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255', 'blank': 'True'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['%s']" % user_orm_label, 'through': "orm['badgify.Award']", 'symmetrical': 'False'}),
            'users_count': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
    }

    complete_apps = ['badgify']



# -*- coding: utf-8 -*-
from django.db.models import signals
from django.test import TestCase

from .. import settings
from .. import commands
from .. import registry
from ..models import Badge, Award
from ..compat import get_user_model

from .recipes import Recipe1, Recipe2


class CommandsTestCase(TestCase):
    """
    Commands test case.
    """

    def setUp(self):
        reload(settings)
        registry.clear()

        self.post_save_count = 0
        self.pre_delete_count = 0

        signals.post_save.connect(self._award_post_save_receiver, sender=Award)
        signals.pre_delete.connect(self._award_pre_delete_receiver, sender=Award)

    def tearDown(self):
        registry.clear()

        signals.post_save.disconnect(self._award_post_save_receiver, sender=Award)
        signals.pre_delete.disconnect(self._award_pre_delete_receiver, sender=Award)

    def _award_post_save_receiver(self, *args, **kwargs):
        self.post_save_count += 1

    def _award_pre_delete_receiver(self, *args, **kwargs):
        self.pre_delete_count += 1

    def test_sync_badges(self):
        registry.register([Recipe1, Recipe2])
        created = commands.sync_badges()
        self.assertEqual(len(created), 2)

    def test_sync_badges_update(self):
        registry.register([Recipe1, Recipe2])
        commands.sync_badges()

        Recipe1.name = 'Howdy'
        commands.sync_badges(update=True)

        self.assertEqual(Badge.objects.get(slug='recipe1').name, 'Howdy')
        Recipe1.name = 'Recipe 1'

    def test_sync_count(self):
        settings.AUTO_DENORMALIZE = False

        user = get_user_model().objects.create_user('user', 'user@example.com', '$ecret')
        registry.register([Recipe1, Recipe2])
        commands.sync_badges()
        updated, unchanged = commands.sync_counts()

        self.assertEqual(len(updated), 0)
        self.assertEqual(len(unchanged), 2)

        Award.objects.create(user=user, badge=Badge.objects.get(slug='recipe1'))
        updated, unchanged = commands.sync_counts()

        self.assertEqual(len(updated), 1)
        self.assertEqual(len(unchanged), 1)

    def test_sync_awards_auto_denormalize_false(self):
        settings.AUTO_DENORMALIZE = False

        user = get_user_model().objects.create_user('user', 'user@example.com', '$ecret')
        registry.register(Recipe1)
        created = commands.sync_badges()
        recipe = registry.get_recipe_instance('recipe1')

        self.assertEqual(len(created), 1)
        self.assertEqual(recipe.badge.users.count(), 0)

        user.love_python = True
        user.save()

        commands.sync_awards()

        self.assertEqual(recipe.badge.users.count(), 1)
        self.assertEqual(recipe.badge.users_count, 0)
        self.assertEqual(self.post_save_count, 1)

    def test_sync_awards_auto_denormalize_true(self):
        settings.AUTO_DENORMALIZE = True

        user = get_user_model().objects.create_user('user1', 'user1@example.com', '$ecret')
        registry.register(Recipe1)
        commands.sync_badges()
        recipe = registry.get_recipe_instance('recipe1')

        user.love_python = True
        user.save()

        commands.sync_awards()

        self.assertEqual(recipe.badge.users.count(), 1)
        self.assertEqual(recipe.badge.users_count, 1)
        self.assertEqual(self.post_save_count, 1)
        self.assertEqual(self.pre_delete_count, 0)

        Award.objects.all().delete()

        self.assertEqual(self.pre_delete_count, 1)
        self.assertEqual(recipe.badge.users.count(), 0)
        self.assertEqual(recipe.badge.users_count, 0)

    def test_sync_awards_disable_signals(self):
        settings.AUTO_DENORMALIZE = True

        user = get_user_model().objects.create_user('user', 'user@example.com', '$ecret')
        registry.register(Recipe1)
        created = commands.sync_badges()
        recipe = registry.get_recipe_instance('recipe1')

        self.assertEqual(len(created), 1)
        self.assertEqual(self.post_save_count, 0)
        self.assertEqual(recipe.badge.users_count, 0)

        user.love_python = True
        user.save()

        commands.sync_awards(**{'disable_signals': True})

        self.assertEqual(recipe.badge.users_count, 0)
        self.assertEqual(self.post_save_count, 0)



# -*- coding: utf-8 -*-
from optparse import make_option

from django.core.management.base import LabelCommand, CommandError

from badgify import commands
from badgify.utils import sanitize_command_options


class Command(LabelCommand):
    """
    Command that synchronizes badges, awards and counts.
    """
    help = u'Synchronizes badges, awards and counts.'

    option_list = LabelCommand.option_list + (

        make_option('--badges',
            action='store',
            dest='badges',
            type='string'),

        make_option('--db-read',
            action='store',
            dest='db_read',
            type='string'),

        make_option('--disable-signals',
            action='store_true',
            dest='disable_signals'),

        make_option('--batch-size',
            action='store',
            dest='batch_size',
            type='int'),

        make_option('--update',
            action='store_true',
            dest='update'),

        make_option('--exclude-badges',
            action='store',
            dest='exclude_badges',
            type='string'),
    )

    def handle_label(self, label, **options):
        options = sanitize_command_options(options)

        if not hasattr(commands, 'sync_%s' % label):
            raise CommandError('"%s" is not a valid command.' % label)

        getattr(commands, 'sync_%s' % label)(**options)



# -*- coding: utf-8 -*-
from optparse import make_option

from django.core.management.base import BaseCommand

from badgify.commands import show_stats


class Command(BaseCommand):
    """
    Commands that shows badge stats.
    """
    help = 'Shows badge stats.'

    option_list = BaseCommand.option_list + (
        make_option('--db-read',
            action='store',
            dest='db_read',
            type='string'),
    )

    def handle(self, *args, **options):
        show_stats(**options)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Award',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('awarded_at', models.DateTimeField(verbose_name='awarded at', auto_now_add=True)),
            ],
            options={
                'abstract': False,
                'verbose_name_plural': 'awards',
                'verbose_name': 'award',
            },
        ),
        migrations.CreateModel(
            name='Badge',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('name', models.CharField(verbose_name='name', help_text='The badge name', max_length=255)),
                ('slug', models.SlugField(verbose_name='slug', unique=True, help_text='The badge slug (auto-generated if empty)', blank=True, max_length=255)),
                ('description', models.TextField(verbose_name='description', blank=True, help_text='The badge description')),
                ('image', models.ImageField(verbose_name='Image', help_text='Please, upload an image for this badge', upload_to='badges', null=True, blank=True)),
                ('users_count', models.IntegerField(editable=False, default=0, verbose_name='users count')),
                ('users', models.ManyToManyField(verbose_name='users', to=settings.AUTH_USER_MODEL, help_text='Users that earned this badge', through='badgify.Award')),
            ],
            options={
                'abstract': False,
                'verbose_name_plural': 'badges',
                'verbose_name': 'badge',
            },
        ),
        migrations.AddField(
            model_name='award',
            name='badge',
            field=models.ForeignKey(verbose_name='badge', to='badgify.Badge', related_name='awards'),
        ),
        migrations.AddField(
            model_name='award',
            name='user',
            field=models.ForeignKey(verbose_name='user', to=settings.AUTH_USER_MODEL, related_name='badges'),
        ),
        migrations.AlterUniqueTogether(
            name='award',
            unique_together=set([('user', 'badge')]),
        ),
    ]



# -*- coding: utf-8 -*-
from django.apps import AppConfig


class BadgifyConfig(AppConfig):
    name = 'badgify'
    verbose_name = 'Badgify'

    def ready(self):
        super(BadgifyConfig, self).ready()
        self.module.autodiscover()



# -*- coding: utf-8 -*-
from django.conf import settings
from django.conf.urls import patterns, include, url
from django.conf.urls.static import static
from django.contrib import admin
from  django.views.generic import TemplateView

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^badges/', include('badgify.urls')),
    url(r'^$', TemplateView.as_view(template_name='home.html'), name='home')
) + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

root = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(root, 'README.rst')) as f:
    README = f.read()

setup(
    name='django-badgify',
    version='0.1.6',
    description='A reusable application to create your own badge engine using Django',
    long_description=README,
    author='Gilles Fabio',
    author_email='gilles.fabio@gmail.com',
    url='http://github.com/ulule/django-badgify',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'Pillow==2.4.0',
        'pytz',
    ],
    tests_require=['coverage', 'RandomWords'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Utilities',
    ]
)



# -*- coding: utf-8 -*-
from optparse import make_option

from django.core.management.base import BaseCommand

from badgify.commands import reset_awards
from badgify.utils import sanitize_command_options


class Command(BaseCommand):
    """
    Commands that resets awards.
    """
    help = 'Resets awards.'

    option_list = BaseCommand.option_list + (

        make_option('--badges',
            action='store',
            dest='badges',
            type='string'),

        make_option('--exclude-badges',
            action='store',
            dest='exclude_badges',
            type='string'),
    )

    def handle(self, *args, **options):
        options = sanitize_command_options(options)
        reset_awards(**options)



# -*- coding: utf-8 -*-
import os
import django

ROOT = os.path.dirname(os.path.abspath(__file__))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': '',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

SITE_ID = 1

DEBUG = True

MIDDLEWARE_CLASSES = ()

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'badgify',
    'badgify.tests',
]

AUTH_USER_MODEL = 'tests.BadgifyUser'

MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(ROOT, 'media')
STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(ROOT, 'static')

SECRET_KEY = 'blabla'

ROOT_URLCONF = 'badgify.urls'

TEMPLATE_DIRS = (
    os.path.join(ROOT, 'templates'),
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'stream': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['stream'],
            'level': 'ERROR',
            'propagate': True,
        },
        'badgify': {
            'handlers': ['stream'],
            'level': 'ERROR',
            'propagate': True,
        },
    },
}

if django.VERSION < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'
if django.VERSION < (1, 7):
    SOUTH_MIGRATION_MODULES = {'badgify': 'badgify.south_migrations'}
    INSTALLED_APPS.append('south')
